import pandas as pd

# Create a dataframe from an excel sheet

filename = 'fires_by_month.xlsx'

df = pd.read_excel(filename, sheet_name='firedata')

print(
    df.info()
)

print(df)

cols_data = ['state','fire_year','fire_count']

df1 = df[cols_data]  # new dataframe = df[specified columns]
print(df1)

# Excel -> group data then aggregate to display
# pandas -> .groupby(colname)[aggcolumm].agg()

groupByState = df1.groupby('state')['fire_count'].sum().reset_index()
# new dataframe = groupby() -> new index

print(groupByState)

# multiple agggregates using the agg() function with a list of aggs()
# sum(), mean(), min(), max() -> agg()

groupByStateStats = df1.groupby('state').agg(
    {
        'fire_count': ['sum','mean','min','max']
    }
).reset_index() # agg()

print(groupByStateStats)

# we can use multiple aggs() for multiple cols
groupByMetrics = df.groupby('state').agg(
    {
        'fire_count': ['sum','mean','min','max'],
        'acres_burned': ['sum','mean','max'],
        'days_burning': ['sum','mean','max'],
        'fire_year': ['min','max']
    }
).round(2)

print(groupByMetrics)

# use an agg() to get a value, then query the agg()
# States with fewer than 500 fires

# option 1 Pythonic
# sum() to get the total number of fires
# relational operator < 500 on the sum
# First sum up the fire counts by state
state_fire_counts = df.groupby('state')['fire_count'].sum()
# Then relational operator
low_fire_counts = state_fire_counts[state_fire_counts < 500]
print(low_fire_counts.sort_values(ascending=True))

# option 2 Python Syntax with pipes
# low_fire_countsOption2 = groupByMetrics[groupByMetrics['fire_count'].sum() < 500].sort_values('fire_count')

# Pivot Table the Fire Data
pivot_data = df.pivot_table(
    # 5 common / base attrs for pivot table
    values='fire_count', # actual value to output where x and y cross
    index='state',
    columns='fire_year',
    aggfunc='sum',
    fill_value=0  # optional but recommended
) # data

# options for pivot tables
pivot_data['Total'] = pivot_data.sum(axis=1)
pivot_data = pivot_data.sort_values('Total', ascending=False)
pivot_data = pivot_data.drop('Total', axis=1)

print(pivot_data)

# filter using a range() or between() function
# range() -> any ranges not involving date/time
# between() -> anything -> Best Practice

# The years between 2000 and 2015
df_2000_2015 = df[
    df['fire_year'].between(2000,2015) # boundaries, not start:end
    ]  # df

print(df_2000_2015)

# Map data to new data using the map()
month_map = {
    1: 'Jan',
    2: 'Feb',
    3: 'Mar',
    4: 'Apr',
    5: 'May',
    6: 'Jun',
    7: 'Jul',
    8: 'Aug',
    9: 'Sep',
    10: 'Oct',
    11: 'Nov',
    12: 'Dec'
}

df_text_months = df_2000_2015.copy()

df_text_months['fire_month'] = df_text_months['fire_month'].map(month_map)

print(df_text_months)


df_missing = pd.read_excel('fires_by_month_missing.xlsx', sheet_name='firedata')

print(
    df_missing.info()
)

# Find the values that are missing
# isna()

missing_data = df_missing[df_missing['fire_count'].isna()]
print(
    missing_data
)

# Get a count of missing values by state
missing_by_state = df_missing['fire_count'].isna().groupby(df['state']).sum()
print(missing_by_state[missing_by_state > 0])

# Create a clean dataframe exluding missing values 
# use numpy to replace the 0 values with NaN
import numpy as np

df_clean = missing_data.copy()

df_clean['fire_count'] = df_clean['fire_count'].replace({
    0: np.nan
})
df_clean = df_clean.dropna(subset=['fire_count'])

print(df_clean)