# imports
import matplotlib.pyplot as plt
import pandas as pd
import numpy as np

# Create a dataframe from housingData.csv
filename = 'housingData.csv'
housing_data = pd.read_csv(filename)

housing_data = housing_data.query('price < 500000')

# A1 - R1601
# row 1 header
# row 2 - 1601  [0-1600]

def myLinePlot():
    '''
    Line Plot
    Purpose -> trending sequence or time series data
    Best Practices -> trending, rate of change, multiple series over same sequence
    Advantages -> Easy to read, looking at continuous change
    Practical Apps -> stock trending, temp changes, sales trends
    '''
    # figure in memory
    plt.figure(figsize=(12,6))
    # data range    kind='line'
    plt.plot(range(len(housing_data)), housing_data['price'], marker='o')
    # outside markers (title, xlabel, ylabel)
    plt.title('House Price Line Plot')
    plt.xlabel('House Index')
    plt.ylabel('Price in Millons ($)')
    plt.grid(True) # default False, so you don't need this function for no grid
    # plt.show() # in notebook programs the show() is handled by the run of the cell, in editor programs you will need the show() to 'get' the plot() from memory

def myAreaPlot():
    '''
    Area Plot (types of line plots with 'fill data')
    Similar to line plots but emphasize overall volume vs static data
    You might not use the marker attr, it may be replaced with alpha attr(ranged 0-1)
    '''
    # figure in memory
    plt.figure(figsize=(12,6))
    # data range    kind='line'
    plt.plot(range(len(housing_data)), housing_data['price'], alpha=0.3)
    # outside markers (title, xlabel, ylabel)
    plt.title('House Price Area Plot')
    plt.xlabel('House Index')
    plt.ylabel('Price in Millons ($)')
    plt.grid(True) # default False, so you don't need this function for no grid
    # plt.show() # in notebook programs the show() is handled by the run of the cell, in editor programs you will need the show() to 'get' the plot() from memory

def myScatterPlot():
    '''
    Scatter Plots
    Purpose -> Show relationships between two variables
    Best Practices -> Identify coorelations, showing outliers, distribution of large amounts of data
    Advantage -> Pattern detection
    Usage -> anytime you use the vs. keyword (price vs. sqft, bedrooms vs bathrooms)
    '''
    plt.figure(figsize=(10,6))
    plt.scatter(housing_data['sqft_living'], housing_data['price'])
    plt.title('Price vs Living Space Scatter Plot')
    plt.xlabel('Sq Ft. Living')
    plt.ylabel('Price')
    # plt.show()

def myBarPlot():
    '''
    Bar Plots
    Purpose -> Compare qty across different categories
    Best Practices -> requency distribution, ranking data
    Advantage -> categorize our data then visualize
    Common Applications -> anytime you use the by keyword (Ave Price by City)
    '''

    plt.figure(figsize=(12,8))
    city_avg_prices = housing_data.groupby('city')['price'].mean() # avg
    plt.bar(city_avg_prices.index, city_avg_prices.values)
    plt.title('Avg House Price by City Bar Plot')
    plt.xlabel('City')
    plt.ylabel('Average Price ($)')
    plt.xticks(rotation=90)
    plt.show()

def myHistogramPlot():
    '''
    Histograms
    Purpose -> show distribution of continuous data
    Best Practices -> utilization of 'bins' to check for commonality/normancy
    Advantage -> data skewness
    Common Apps -> Test scoring
    '''
    plt.figure(figsize=(10,6))
    plt.hist(housing_data['price'], bins=5, edgecolor='black')
    plt.title('Distribution of House Prices')
    plt.xlabel('Price ($)')
    plt.ylabel('Distribution Frequency')
    plt.show()

def myBoxPlot():
    '''
    Purpose -> show distribution of summary data
    Best Practices -> statistical data analysis
    Advantages -> The ability to show multiple elements at the same time (median, quartiles, outlier)
    Common Apps -> anytime you use the by keyword when distributing data
    Price distribution by City
    Variations in SqFt by Rooms    
    '''
    plt.figure(figsize=(12,6))
    plt.boxplot([
        housing_data[housing_data['city'] == city]['price'] # 2 series from dataframe
        for city in housing_data['city'].unique()
        ],  # x param1 []
        labels=housing_data['city'].unique()
        ) # boxplot()
    plt.title('House Price Distribution by City')
    plt.ylabel('Price ($)')
    plt.xticks(rotation=90)
    plt.show()

def myPiePlot():
    '''
    Purpose -> show a 100% composition/composite of a whole data set
    Best Practices -> All about the percentage of value
    Advantage -> Visualization of SMALLER number of categories
    Common Apps -> # of sales as percentage, market share percentages, budget/sales allocation, survey results as percentage. When you use the as percentage keyword with SMALLER number of categories
    '''
    plt.figure(figsize=(10,10))
    bedroom_counts = housing_data['bedrooms'].value_counts()
    plt.pie(bedroom_counts.values, labels=bedroom_counts.index, autopct='%1.1f%%', startangle=90)
    labels = bedroom_counts.index
    percentages = bedroom_counts.values
    plt.title('Distribution of Houses by Number of Bedrooms')
    legend_labels = [
        f'# Bedrooms: {label} # Houses: {pct}' for label, pct in zip(labels, percentages)
    ]
    plt.legend(legend_labels, title='Disctribution Data', loc="upper right")
    plt.show()

# Call the function for the plot you want to see

