# imports
import pandas as pd # data frame
import numpy as np
import seaborn as sns # regplot()
import matplotlib.pyplot as plt # title(), x and y label(), show()
from sklearn.linear_model import LinearRegression # data modeling class

'''
Housing Data Linear and Multi Regression Models
1. Price predictions based on square feet 'Price'
2. Residual Analysis "appropriate or accurate model / % of error" 'Price'
3. Relationship Analysis -> What is the relationship between multiple features
4. Model Comparisions -> compare a single reg model -> multi reg models

Parameters Regression Models
alpha=<value between 0 and 1> -> sets of transparent data to plot() 0(fully transparent) 1(fully opaque) <0.7>
best practices -> .7 scatter plots (density in data), .3 bar/line/pie (the most density of data) 1 very little data/sparse

LinearRegression Class -> LinearRegression() Constructor -> construct model

Data Modeling (test/train)
simple_model.fit() -> train the data  x,y
simple_model.predict() -> predictions based on the trained data -> array
residual =  y_test.model - predicted.model

Best Practices Regression Models
1. Always split data into both test and trained set
2. Set the alpha attr and change as appropriate
3. Validation of data model using the original set
4. Create multiple subplots to compare/visualize the outputs
'''

filename = 'housingData.csv'
df = pd.read_csv(filename)

# Plot 1 Basic Price vs Square Footage
def simple_linear_regression():
    '''
    1. Relationship between price and square feet
    2. Line will be the regression model / scatter plot data
    3. Confidence interval (alpha)
    4. Outliers (potential)
    '''
    plt.figure(figsize=(10,6))
    sns.regplot(data=df, x='sqft_living', y='price', scatter_kws={'alpha':0.7}, line_kws={'color': 'red'})
    plt.title('House Prices vs Living Space')
    plt.xlabel('Square Feet of Living Space')
    plt.ylabel('Price')
    plt.show()

# Plot 2 Residual Analysis
def residual_analysis():
    '''
    1. How far/what is the difference of actual price from predicted price
    2. Determine if our simple_linear_regression() is / was accurate
    3. You MUST use the same alpha as the initial regression
    3. Can we determine where any errors are happening in data model
     
    '''
    model = LinearRegression()
    X = df[['sqft_living']]
    y = df['price']
    model.fit(X,y)
    residuals = y - model.predict(X) # 250000 - model.predicted(sqft->300000)

    plt.figure(figsize=(10,6))
    plt.scatter(df['sqft_living'], residuals, alpha=0.7)
    plt.axhline(y=0, color='r', linestyle='--')
    plt.title('Residuals vs Living Space')
    plt.xlabel('Square Feet of Living Space')
    plt.ylabel('Residual (Actual - Predicted Price)')
    plt.show()

# Plot 3 Multiple Plots Feature Relationships
def feature_realtionships():
    '''
    1. How different features affect each other
    2. correlations or ratios of affect between features
    3. What features might be most usedful for predictive analysis
    '''
    plt.figure(figsize=(12,4))
    plt.subplot(1,3,1)
    plt.scatter(df['sqft_living'], df['bedrooms'], alpha=0.7)
    plt.title("Living Space vs Bedrooms")

    plt.subplot(1,3,2)
    plt.scatter(df['sqft_living'], df['bathrooms'], alpha=0.7)
    plt.title("Living Space vs Bathrooms")

    plt.subplot(1,3,3)
    plt.scatter(df['bedrooms'], df['bathrooms'])
    plt.title("Bedrooms vs Bathrooms")

    plt.tight_layout()
    plt.show()

# Plot 4 Model Comparisons simple vs multi reg model
def model_comparisons():
    '''
    Simple -> sqft_living
    Multi -> sqft_living, bedrooms, bathrooms
    How well do these predict price
    '''
    # Linear Regression Models
    simple_model = LinearRegression()
    multi_model = LinearRegression()

    X_simple = df[['sqft_living']]
    X_multi = df[['sqft_living','bedrooms', 'bathrooms']]
    y = df['price']

    simple_model.fit(X_simple, y)
    multi_model.fit(X_multi, y)

    # Get predicted price affect
    simple_pred = simple_model.predict(X_simple)
    multi_pred = multi_model.predict(X_multi)

    # 2 subplots() 1. simple 2. multi
    plt.figure(figsize=(12,6))

    plt.subplot(1,2,1)
    plt.scatter(y/1000000, simple_pred/1000000, alpha=0.7)
    r = [y.min()/1000000, y.max()/1000000] # x and y axis
    plt.plot(r,r,'r--')
    plt.title('Simple Regression using Sq. Feet')
    plt.xlabel('Actual $ (millions)')
    plt.ylabel('Predicted $ (millions)')

    plt.subplot(1,2,2)
    plt.scatter(y/1000000, multi_pred/1000000, alpha=0.7)
    r = [y.min()/1000000, y.max()/1000000] # x and y axis
    plt.plot(r,r,'r--')
    plt.title('Multi Regression using Sq. Feet, Bedrooms and Bathrooms')
    plt.xlabel('Actual $ (millions)')
    plt.ylabel('Predicted $ (millions)')
    plt.show()

    plt.tight_layout()
    plt.show()

model_comparisons()